@@ -1,33 +1,47 @@
-# Copyright 2001-3 by Roger Bivand 
+# Copyright 2001-4 by Roger Bivand 
 #
 
-moran <- function(x, listw, n, S0, zero.policy=FALSE) {
-	z <- scale(x, scale=FALSE)
-	zz <- sum(z^2)
-	K <- (length(x)*sum(z^4))/(zz^2)
-	lz <- lag.listw(listw, z, zero.policy=zero.policy)
-	I <- (n / S0) * ((t(z) %*% lz) / zz)
+moran <- function(x, listw, n, S0, zero.policy=FALSE, NAOK=FALSE) {
+	n1 <- length(listw$neighbours)
+	if (n1 != length(x)) stop("objects of different length")
+	xx <- mean(x, na.rm=NAOK)
+	z <- x - xx
+	zz <- sum(z^2, na.rm=NAOK)
+	K <- (length(x)*sum(z^4, na.rm=NAOK))/(zz^2)
+	lz <- lag.listw(listw, z, zero.policy=zero.policy, NAOK=NAOK)
+#	I <- (n / S0) * ((t(z) %*% lz) / zz)
+	I <- (n / S0) * ((sum(z*lz, na.rm=NAOK)) / zz)
 	res <- list(I=I, K=K)
 	res
 }
 
 moran.test <- function(x, listw, randomisation=TRUE, zero.policy=FALSE,
-	alternative="greater", rank = FALSE, spChk=NULL) {
+	alternative="greater", rank = FALSE, na.action=na.fail, spChk=NULL) {
 	alternative <- match.arg(alternative, c("greater", "less", "two.sided"))
 	if (!inherits(listw, "listw")) stop(paste(deparse(substitute(listw)),
 		"is not a listw object"))
 	if (!is.numeric(x)) stop(paste(deparse(substitute(x)),
 		"is not a numeric vector"))
-	if (any(is.na(x))) stop("NA in X")
-	n <- length(listw$neighbours)
-	if (n != length(x)) stop("objects of different length")
 	if (is.null(spChk)) spChk <- get.spChkOption()
 	if (spChk && !chkIDs(x, listw))
 		stop("Check of data and weights ID integrity failed")
+#	if (any(is.na(x))) stop("NA in X")
+	xname <- deparse(substitute(x))
+	wname <- deparse(substitute(listw))
+	NAOK <- deparse(substitute(na.action)) == "na.pass"
+	x <- na.action(x)
+	na.act <- attr(x, "na.action")
+	if (!is.null(na.act)) {
+	    subset <- !(1:length(listw$neighbours) %in% na.act)
+	    listw <- subset(listw, subset, zero.policy=zero.policy)
+	}
+	n <- length(listw$neighbours)
+	if (n != length(x)) stop("objects of different length")
 	
 	wc <- spweights.constants(listw, zero.policy=zero.policy)
 	S02 <- wc$S0*wc$S0
-	res <- moran(x, listw, wc$n, wc$S0, zero.policy=zero.policy)
+	res <- moran(x, listw, wc$n, wc$S0, zero.policy=zero.policy, 
+		NAOK=NAOK)
 	I <- res$I
 	K <- res$K
 	if (rank) K <- (3*(3*wc$n^2 -7))/(5*(wc$n^2 - 1))
@@ -55,29 +69,41 @@ moran.test <- function(x, listw, randomisation=TRUE, zero.policy=FALSE,
 	names(vec) <- c("Moran I statistic", "Expectation", "Variance")
 	method <- paste("Moran's I test under", ifelse(randomisation,
 	    "randomisation", "normality"))
-	data.name <- paste(deparse(substitute(x)), ifelse(rank,
+	data.name <- paste(xname, ifelse(rank,
 		"using rank correction",""), "\nweights:",
-		deparse(substitute(listw)), "\n")
+		wname, ifelse(is.null(na.act), "", paste("\nomitted:", 
+	    paste(na.act, collapse=", "))), "\n")
 	res <- list(statistic=statistic, p.value=PrI, estimate=vec, 
 	    alternative=alternative, method=method, data.name=data.name)
+	if (!is.null(na.act)) attr(res, "na.action") <- na.act
 	class(res) <- "htest"
 	res
 }
 
 moran.mc <- function(x, listw, nsim, zero.policy=FALSE,
-	alternative="greater", spChk=NULL) {
+	alternative="greater", na.action=na.fail, spChk=NULL) {
 	alternative <- match.arg(alternative, c("greater", "less"))
 	if(!inherits(listw, "listw")) stop(paste(deparse(substitute(listw)),
 		"is not a listw object"))
 	if(!is.numeric(x)) stop(paste(deparse(substitute(x)),
 		"is not a numeric vector"))
 	if(missing(nsim)) stop("nsim must be given")
-	if (any(is.na(x))) stop("NA in X")
-	n <- length(listw$neighbours)
-	if (n != length(x)) stop("objects of different length")
 	if (is.null(spChk)) spChk <- get.spChkOption()
 	if (spChk && !chkIDs(x, listw))
 		stop("Check of data and weights ID integrity failed")
+#	if (any(is.na(x))) stop("NA in X")
+	xname <- deparse(substitute(x))
+	wname <- deparse(substitute(listw))
+	if (deparse(substitute(na.action)) == "na.pass")
+	    stop("na.pass not permitted")
+	x <- na.action(x)
+	na.act <- attr(x, "na.action")
+	if (!is.null(na.act)) {
+	    subset <- !(1:length(listw$neighbours) %in% na.act)
+	    listw <- subset(listw, subset, zero.policy=zero.policy)
+	}
+	n <- length(listw$neighbours)
+	if (n != length(x)) stop("objects of different length")
 	if(nsim > gamma(n+1)) stop("nsim too large for this n")
 	
 	S0 <- Szero(listw)
@@ -100,12 +126,14 @@ moran.mc <- function(x, listw, nsim, zero.policy=FALSE,
 	parameter <- xrank
 	names(parameter) <- "observed rank"
 	method <- "Monte-Carlo simulation of Moran's I"
-	data.name <- paste(deparse(substitute(x)), "\nweights:",
-	    deparse(substitute(listw)), "\nnumber of simulations + 1:",
+	data.name <- paste(xname, "\nweights:",
+	    wname, ifelse(is.null(na.act), "", paste("\nomitted:", 
+	    paste(na.act, collapse=", "))), "\nnumber of simulations + 1:",
 	    nsim+1, "\n")
 	lres <- list(statistic=statistic, parameter=parameter,
 	    p.value=pval, alternative=alternative, method=method, 
 	    data.name=data.name, res=res)
+	if (!is.null(na.act)) attr(lres, "na.action") <- na.act
 	class(lres) <- c("htest", "mc.sim")
 	lres
 }
